#
# i2cliquidcrystal.py
#
# Class to control LCD using WebIOPi
# 
# Written by Nobuki HIRAMINE
# http://www.hiramine.com
# Last modified : 2017/05/01
#

import webiopi
from webiopi.devices.i2c import I2C

class I2CLiquidCrystal(I2C) :
	_numlines = 2

	def __init__(self, i2caddr_lcd ):
		# Init I2C
		I2C.__init__(self, i2caddr_lcd)

		# Function Set
		# 2進数表現で、001L NF00 形式のデータを送信する。
		# L : データ長。1 = 8ビット, 0 = 4ビット。AQM0802Aは8bitなので1。
		# N : Line数。1 = 2行。0 = 1行。AQM0802Aは2行なので1。
		# F : フォントタイプ。1 = 5x10ドット, 0 = 5x8ドット。AQM0802Aは5x8ドットなので0。
		# 以上により、2進数表現で、0011 1000。16進数表現で、0x38のデータを送信する。
		self._command( 0x38 )

		# 拡張コマンドに移行する（内部発振周波数、コントラスト等を設定するため）
		# Function Setで送ったデータの1ビット目を立てたデータを送る
		# 0x38を送った場合は、1足して、0x39を送る
		self._command( 0x39 )

		# 内部発振周波数
		# 2進数表現で、0001 [BS][F2][F1][F0] 形式のデータを送信する。
		# データシートより、2進数表現で、0001 0100。16進数表現で、0x14のデータを送信する。
		self._command( 0x14 )

		# コントラスト（1/2）
		# 2進数表現で、0111 [C3][C2][C1][C0] 形式のデータを送信する。
		# データシートより、「3Vの場合、C5=1,C4=0,C3=0,C2=0,C1=0,C0=0程度」
		# 2進数表現で、0111 0000。16進数表現で、0x70のデータを送信する。
		self._command( 0x70 )

		# コントラスト（2/2）
		# 2進数表現で、0101 [Ion][Bon][C5][C4] 形式のデータを送信する。
		# Ion : ICON display on/off。1 = on, 0 = off。Offとしたいので0。
		# Bon : Booster circuit on/off。3.3Vの場合はBooster回路をOnとするので1。
		# データシートより、「3Vの場合、C5=1,C4=0,C3=0,C2=0,C1=0,C0=0程度」
		# 2進数表現で、0101 0110。16進数表現で、0x56のデータを送信する。
		self._command( 0x56 )

		# Follower control
		# 2進数表現で、0110 [Fon][Rab2][Rab1][Rab0] 形式のデータを送信する。
		# データシートより、2進数表現で、0110 1100。16進数表現で、0x6Cのデータを送信する。
		self._command( 0x6C )

		# 500msec(need 200msec : for power stable)
		webiopi.sleep( 0.500 )

		# 拡張コマンドの終了（内部発振周波数、コントラスト等を設定するため）
		# Function Setで送ったデータの1ビット目を倒したデータを送る
		self._command( 0x38 )

		# Display ON/OFF Control
		# 2進数表現で、0000 1DCB 形式のデータを送信する。
		# D : Display On/Off。1 = On, 0 = Off。Display On なので1。
		# C : Cursor On/Off。1 = On, 0 = Off。Cursor Offとしたいので0。
		# B : Blinking of cursor On/Off。1 = On, 0 = Off。Blinking Offとしたいので0。
		# 以上により、2進数表現で、0000 1100。16進数表現で、0x0Cのデータを送信する。
		self._command(0x0C);

		# Entry Mode Set
		# 2進数表現で、0000 01DS 形式のデータを送信する。
		# D : Cursor move direction。1 = Increment, 0 = Decrement。カーソル移動方向は増加方向なので1。
		# S : Shift the display。1 = Shiftする, 0 = Shiftしない。Shiftしないとしたいので0。
		# 以上により、2進数表現で、0000 0110。16進数表現で、0x06のデータを送信する。
		self._command(0x06);

		self.clear()			# Clear display

	def clear( self ):
		self._command( 0x01 )
		webiopi.sleep( 0.002 )		# 2msec(need 1.06msec)

	def setCursor( self, col, row ):
		row_offsets = [0x00, 0x40]
		if row >= self._numlines:
			row = self._numlines - 1
		self._command( 0x80 | ( row_offsets[row] + col ) )

	def print( self, value ):
		for i in range( len(value) ):
			self._send( ord(value[i]), True )

	def _command( self, value ):
		self._send( value, False )
		webiopi.sleep( 0.000100 )	# 100microsec(need 26.3microsec)

	def _send( self, value, mode ):
		if mode:	# print mode
			I2C.writeRegister( self, 0x40, value )
		else:		# command mode
			I2C.writeRegister( self, 0x00, value )
